Exception: Canon::SizeLimitExceededError
- Defined in:
- lib/canon/errors.rb
Overview
Error raised when input exceeds size limits
This error is raised when input files or trees exceed configured size limits to prevent performance issues or hangs.
Instance Attribute Summary collapse
-
#actual ⇒ Object
readonly
Returns the value of attribute actual.
-
#limit ⇒ Object
readonly
Returns the value of attribute limit.
-
#limit_type ⇒ Object
readonly
Returns the value of attribute limit_type.
Instance Method Summary collapse
-
#initialize(limit_type, actual, limit) ⇒ SizeLimitExceededError
constructor
Initialize a new SizeLimitExceededError.
Constructor Details
#initialize(limit_type, actual, limit) ⇒ SizeLimitExceededError
Initialize a new SizeLimitExceededError
70 71 72 73 74 75 |
# File 'lib/canon/errors.rb', line 70 def initialize(limit_type, actual, limit) @limit_type = limit_type @actual = actual @limit = limit super() end |
Instance Attribute Details
#actual ⇒ Object (readonly)
Returns the value of attribute actual.
62 63 64 |
# File 'lib/canon/errors.rb', line 62 def actual @actual end |
#limit ⇒ Object (readonly)
Returns the value of attribute limit.
62 63 64 |
# File 'lib/canon/errors.rb', line 62 def limit @limit end |
#limit_type ⇒ Object (readonly)
Returns the value of attribute limit_type.
62 63 64 |
# File 'lib/canon/errors.rb', line 62 def limit_type @limit_type end |